"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDataBinds = exports.createEventHanlders = exports.createWidgetProps = exports.generateDataBind = exports.generatedDynamicData = exports.extractWidgetProps = void 0;
const weapps_core_1 = require("../../weapps-core");
const weapp_1 = require("../util/weapp");
const wxml_1 = require("./wxml");
const mp_1 = require("../config/mp");
function extractWidgetProps(props) {
    var _a, _b;
    const { classList } = props;
    const staticProps = {
        style: (0, weapps_core_1.toCssStyle)(props.commonStyle, {
            toRpx: true,
            toRem: false,
        }),
        classList: classList || [],
    };
    const { data = {} } = props;
    Object.assign(staticProps, generatedDynamicData(data).staticProps);
    if ((_a = props.directives) === null || _a === void 0 ? void 0 : _a.waForKey) {
        staticProps._waForKey = (_b = props.directives) === null || _b === void 0 ? void 0 : _b.waForKey.value;
    }
    return staticProps;
}
exports.extractWidgetProps = extractWidgetProps;
function generatedDynamicData(data) {
    const staticProps = {};
    const boundProps = {};
    for (const key in data) {
        if (mp_1.builtinWigetProps.indexOf(key) > -1)
            continue;
        const { type, value } = data[key];
        if (!type || type === 'static') {
            staticProps[key] = value;
        }
        else {
            boundProps[key] = generateDataBind(data[key]);
        }
    }
    return { staticProps, boundProps };
}
exports.generatedDynamicData = generatedDynamicData;
function generateDataBind(bind) {
    let expr = '';
    const { type, value } = bind;
    if (type === weapps_core_1.PropBindType.forItem) {
        expr = `forItems.${value}`;
    }
    else if (type === weapps_core_1.PropBindType.expression) {
        expr = `${value.replace(/\$comp./g, `${weapps_core_1.COMPONENT_API_PREFIX}.`)}`;
    }
    else if (type === weapps_core_1.PropBindType.prop) {
        const isNegated = value.startsWith('!');
        const originValue = value.replace(/^!/, '');
        expr = `${isNegated ? '!' : ''}${weapps_core_1.COMPONENT_API_PREFIX}.props.data.${originValue}`;
    }
    else {
        const isGlobal = value.startsWith('global.');
        const isCopositive = value.startsWith('comp-') || value.startsWith('$comp_');
        const PREFIX_MAP = {
            [weapps_core_1.PropBindType.state]: 'state',
            [weapps_core_1.PropBindType.computed]: 'computed',
            [weapps_core_1.PropBindType.dataVar]: 'dataVar',
            [weapps_core_1.PropBindType.stateData]: 'dataset.state',
            [weapps_core_1.PropBindType.paramData]: 'dataset.params',
        };
        switch (type) {
            case weapps_core_1.PropBindType.state:
            case weapps_core_1.PropBindType.computed:
            case weapps_core_1.PropBindType.dataVar:
            case weapps_core_1.PropBindType.stateData:
            case weapps_core_1.PropBindType.paramData: {
                if (isCopositive) {
                    expr = value
                        .replace(/^comp-\w+/, `${weapps_core_1.COMPONENT_API_PREFIX}.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `${weapps_core_1.COMPONENT_API_PREFIX}.${PREFIX_MAP[type]}`);
                }
                else {
                    expr = value
                        .replace(/^\w+./, `${isGlobal ? 'app' : '$page'}.${PREFIX_MAP[type]}.`)
                        .replace(/^comp-\w+/, `${weapps_core_1.COMPONENT_API_PREFIX}.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `${weapps_core_1.COMPONENT_API_PREFIX}.${PREFIX_MAP[type]}`);
                }
            }
        }
    }
    return expr;
}
exports.generateDataBind = generateDataBind;
function createWidgetProps(widgets, ctx) {
    const widgetProps = {};
    (0, weapp_1.walkThroughWidgets)(widgets, (id, widget, parentId) => {
        var _a;
        const { xComponent } = widget;
        const xProps = widget.xProps || {};
        if (!xComponent) {
            return;
        }
        const materialLib = ctx.materialLibs.find((lib) => lib.name === xComponent.moduleName);
        const miniprogramPlugin = (_a = ctx.miniprogramPlugins) === null || _a === void 0 ? void 0 : _a.find((plugin) => plugin.name === xComponent.moduleName);
        if (!materialLib && !miniprogramPlugin) {
            console.error('Component lib not found', xComponent);
            return;
        }
        widgetProps[id] = extractWidgetProps(xProps);
        widgetProps[id]._parentId = parentId;
        widgetProps[id]._order = widget.xIndex;
        widgetProps[id].widgetType = xComponent.moduleName + ':' + xComponent.name;
    });
    return widgetProps;
}
exports.createWidgetProps = createWidgetProps;
function createEventHanlders(widgets, componentApi, ctx) {
    const eventHanlders = {};
    (0, weapp_1.walkThroughWidgets)(widgets, (id, widget, parentId) => {
        var _a, _b, _c;
        const { xComponent } = widget;
        const xProps = widget.xProps || {};
        if (!xComponent) {
            return;
        }
        const materialLib = ctx.materialLibs.find((lib) => lib.name === xComponent.moduleName);
        const miniprogramPlugin = (_a = ctx.miniprogramPlugins) === null || _a === void 0 ? void 0 : _a.find((plugin) => plugin.name === xComponent.moduleName);
        if (!materialLib && !miniprogramPlugin) {
            console.error('Component lib not found', xComponent);
            return;
        }
        const compProto = materialLib
            ? materialLib.components.find((comp) => comp.name === xComponent.name)
            : (_b = miniprogramPlugin === null || miniprogramPlugin === void 0 ? void 0 : miniprogramPlugin.componentConfigs) === null || _b === void 0 ? void 0 : _b.find((comp) => comp.name === xComponent.name);
        if (!compProto) {
            return;
        }
        const listeners = (xProps.listeners || []).slice();
        const { inputProps, syncProps } = compProto.meta || {};
        const syncConfigs = syncProps || inputProps || {};
        for (const valuProp in syncConfigs) {
            const config = syncConfigs[valuProp];
            const configs = Array.isArray(config) ? config : [config];
            configs.forEach(({ changeEvent, valueFromEvent = 'event.detail.value' }) => {
                listeners.unshift({
                    trigger: changeEvent,
                    handler: {
                        moduleName: '',
                        name: `function({ event }) { mpCompToWidget(this, event.currentTarget).${valuProp} = ${valueFromEvent} }`,
                    },
                    type: weapps_core_1.ActionType.Inline,
                    data: {},
                });
            });
        }
        listeners.forEach((l) => {
            var _a, _b, _c, _d;
            const handlerName = (0, wxml_1.getMpEventHanlderName)(id, l.trigger, l);
            eventHanlders[handlerName] = eventHanlders[handlerName] || [];
            const params = generatedDynamicData(l.data);
            let handler = (_a = l.handler) === null || _a === void 0 ? void 0 : _a.name;
            switch (l.type) {
                case weapps_core_1.ActionType.Platform: {
                    let method = (_b = l.handler) === null || _b === void 0 ? void 0 : _b.name;
                    handler = `function({data}){ return app.${method}(data)}`;
                    break;
                }
                case weapps_core_1.ActionType.DataSource: {
                    handler = `function({data}){ return app.cloud.callDataSource(data)}`;
                    break;
                }
                case weapps_core_1.ActionType.PropEvent: {
                    handler = `function({event, data = {}}){ return ${componentApi}.props.events.${(_c = l.handler) === null || _c === void 0 ? void 0 : _c.name}({...event.detail, ...data}) }`;
                    break;
                }
            }
            eventHanlders[handlerName].push({
                key: l.key || '',
                handler: handler,
                handlerModule: (_d = l.handler) === null || _d === void 0 ? void 0 : _d.moduleName,
                data: params.staticProps,
                boundData: params.boundProps,
                type: l.type,
            });
        });
        if ((_c = compProto === null || compProto === void 0 ? void 0 : compProto.compConfig) === null || _c === void 0 ? void 0 : _c.isDataContainer) {
            const customName = (0, wxml_1.getMpEventHanlderName)(id, 'onDataChange', {});
            eventHanlders[customName] = [
                {
                    key: `wa${Date.now().toString().slice(-8)}`,
                    handler: `({event}) => {
          app.utils.set(context, '${id}.data', event?.detail?.data);
        }`,
                    handlerModule: weapps_core_1.ActionType.Platform,
                    data: {},
                    boundData: {},
                },
            ];
        }
    });
    return eventHanlders;
}
exports.createEventHanlders = createEventHanlders;
function createDataBinds(widgets, ctx) {
    const dataBinds = {};
    (0, weapp_1.walkThroughWidgets)(widgets, (id, widget, parentId) => {
        var _a;
        const { xComponent } = widget;
        const xProps = widget.xProps || {};
        if (!xComponent) {
            return;
        }
        const materialLib = ctx.materialLibs.find((lib) => lib.name === xComponent.moduleName);
        const miniprogramPlugin = (_a = ctx.miniprogramPlugins) === null || _a === void 0 ? void 0 : _a.find((plugin) => plugin.name === xComponent.moduleName);
        if (!materialLib && !miniprogramPlugin) {
            console.error('Component lib not found', xComponent);
            return;
        }
        dataBinds[id] = {};
        Object.entries(xProps.data || {}).map(([prop, val]) => setDataBind(dataBinds[id], prop, val));
        const { directives = {} } = xProps;
        setDataBind(dataBinds[id], '_waFor', directives.waFor);
        setDataBind(dataBinds[id], '_waIf', directives.waIf);
        setDataBind(dataBinds[id], 'classList', xProps.classListBind);
        setDataBind(dataBinds[id], 'style', xProps.styleBind);
        const { classList, style } = dataBinds[id];
        if (classList) {
            dataBinds[id].classList = `concatClassList(${classList}, widgetProps.${id}.classList)`;
        }
        if (style) {
            dataBinds[id].style = `px2rpx({...widgetProps.${id}.style, ...(${style})})`;
        }
        if (Object.keys(dataBinds[id]).length === 0) {
            delete dataBinds[id];
        }
    });
    return dataBinds;
}
exports.createDataBinds = createDataBinds;
function setDataBind(target, prop, val) {
    if (val && val.type && val.type !== 'static') {
        const jsExpr = generateDataBind(val);
        if (jsExpr) {
            target[prop] = jsExpr;
        }
    }
}
