"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCompLibs = exports.generateCode = exports.deserialize = exports.serialize = void 0;
const generate_1 = require("./generate");
const material_1 = require("./material");
const template_1 = __importDefault(require("../template"));
const path_1 = __importDefault(require("path"));
const types_1 = require("../../types");
var weapps_core_1 = require("../../weapps-core");
Object.defineProperty(exports, "serialize", { enumerable: true, get: function () { return weapps_core_1.serialize; } });
Object.defineProperty(exports, "deserialize", { enumerable: true, get: function () { return weapps_core_1.deserialize; } });
function generateCode(props) {
    return __awaiter(this, void 0, void 0, function* () {
        const { mainAppData, subAppDataList = [], dependencies, appKey = 'test', i18nConfig = {
            enabled: false,
            i18nData: {},
        }, extraData = {
            isComposite: false,
            compProps: {},
        }, basename = '/', isSandbox = false, buildTypeList = ["web"], ignoreInstall = false, deployMode = types_1.DEPLOY_MODE.PREVIEW, runtime = types_1.RUNTIME.NONE, } = props;
        const fileCodeMap = {};
        const appBuildDir = '/';
        const excludeRegex = isSandbox ? /(monitor-jssdk\.min\.js)$/ : null;
        const includeRegex = /^\/src\/(utils|handlers|app|store)/;
        copyFiles({
            appBuildDir,
            excludeRegex,
            includeRegex,
            fileCodeMap,
        });
        if (!isSandbox) {
            yield (0, material_1.runHandleMaterial)(appBuildDir, dependencies, i18nConfig, fileCodeMap, isSandbox);
        }
        yield (0, generate_1.runGenerateCore)({
            appBuildDir,
            appData: mainAppData,
            subAppDataList,
            dependencies,
            appKey,
            buildTypeList,
            basename,
            extraData,
            i18nConfig,
            fileCodeMap,
            isSandbox,
            ignoreInstall,
            deployMode,
            runtime,
        });
        return fileCodeMap;
    });
}
exports.generateCode = generateCode;
function generateCompLibs(compLibs) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileCodeMap = {};
        const dependencies = compLibs;
        const isSandbox = true;
        const i18nConfig = {};
        const appBuildDir = '/';
        yield (0, material_1.runHandleMaterial)(appBuildDir, dependencies, i18nConfig, fileCodeMap, isSandbox);
        const includeRegex = /^\/src\/(utils|handlers)/;
        copyFiles({
            appBuildDir,
            excludeRegex: null,
            includeRegex,
            fileCodeMap,
        });
        return fileCodeMap;
    });
}
exports.generateCompLibs = generateCompLibs;
function copyFiles({ appBuildDir, excludeRegex, includeRegex, fileCodeMap }) {
    Object.keys(template_1.default).map((filePath) => {
        if (includeRegex) {
            const isInclude = includeRegex.test(filePath);
            if (isInclude) {
                const isExclude = excludeRegex === null || excludeRegex === void 0 ? void 0 : excludeRegex.test(filePath);
                if (isExclude) {
                    return;
                }
                const dstFile = path_1.default.join(appBuildDir, filePath);
                fileCodeMap[dstFile] = {
                    code: template_1.default[filePath].code,
                };
            }
        }
    });
}
exports.default = generateCode;
